<?php

namespace App;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use App\Company;


/**
 * App\Membership
 *
 * @property int $id
 * @property int $company_id
 * @property string $firstname
 * @property string $lastname
 * @property string|null $phone_number
 * @property string|null $phone_number_2
 * @property int|null $district_id
 * @property int|null $sector_id
 * @property int|null $cell_id
 * @property int|null $village_id
 * @property int|null $category_id
 * @property int|null $status_id
 * @property int|null $church_register_page
 * @property int|null $small_group_id
 * @property \Illuminate\Support\Carbon|null $created_at
 * @property \Illuminate\Support\Carbon|null $updated_at
 * @property \Illuminate\Support\Carbon|null $membership_date
 * @property string|null $origin_church
 * @property string|null $observation
 * @property string|null $employment_status
 * @property string|null $marital_status
 * @property-read \App\Category|null $categories
 * @property-read \App\Cell|null $cells
 * @property-read Company $companies
 * @property-read \App\District|null $districts
 * @property-read \App\Sector|null $sectors
 * @property-read \App\Small_group|null $smallgroups
 * @property-read \App\Status|null $statuses
 * @property-read \App\Village|null $villages
 * @method static \Illuminate\Database\Eloquent\Builder|Membership newModelQuery()
 * @method static \Illuminate\Database\Eloquent\Builder|Membership newQuery()
 * @method static \Illuminate\Database\Eloquent\Builder|Membership query()
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereCategoryId($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereCellId($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereChurchRegisterPage($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereCompanyId($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereCreatedAt($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereDistrictId($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereEmploymentStatus($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereFirstname($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereId($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereLastname($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereMaritalStatus($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereMembershipDate($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereObservation($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereOriginChurch($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership wherePhoneNumber($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership wherePhoneNumber2($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereSectorId($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereSmallGroupId($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereStatusId($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereUpdatedAt($value)
 * @method static \Illuminate\Database\Eloquent\Builder|Membership whereVillageId($value)
 * @mixin \Eloquent
 */
class Membership extends Model
{
    use HasFactory;
    protected $fillable = ['company_id','firstname','lastname','phone_number','district_id','sector_id','village_id','category','status_id','church_register_page','small_group_id'];


    protected $casts = ['membership_date'=>'date'];

    public function companies(){

        return $this->belongsTo(Company::class,'company_id');
    }

    public function districts(){
        return $this->belongsTo(District::class,'district_id');
    }

    public function sectors(){
        return $this->belongsTo(Sector::class,'sector_id');
    }

    public function cells(){
        return $this->belongsTo(Cell::class,'cell_id');
    }

    public function villages(){
        return $this->belongsTo(Village::class,'village_id');
    }

    public function categories(){
        return $this->belongsTo(Category::class,'category_id');
    }


    public function statuses(){
        return $this->belongsTo(Status::class,'status_id');
    }

    public function smallgroups(): \Illuminate\Database\Eloquent\Relations\BelongsTo
    {
        return $this->belongsTo(Small_group::class,'small_group_id');
    }



}
