<?php

use App\Http\Controllers\ExpenseCategoryController;
use App\Http\Controllers\ExpenseController;
use App\Http\Controllers\IndicatorDefinitionController;
use App\Http\Controllers\MembershipController;
use App\Http\Controllers\OfferingTypeController;
use App\Http\Controllers\PaymentOptionController;
use App\Http\Controllers\ReportDefinitionController;
use App\Http\Controllers\ReportingDataController;
use App\Http\Controllers\ReportsController;
use App\Http\Controllers\SmallGroupController;
use App\Http\Controllers\TitheAndOfferingController;
use App\Http\Controllers\TitheOfferingBaseController;
use App\TitheOfferingBase;
use Illuminate\Support\Facades\Route;

Route::get('/optimize', function () {
    Artisan::call('optimize:clear');
    return "Optimized!";
});

Route::middleware(['auth'])->group(function () {

    Route::get('/', 'DashboardController@index');
    Route::get('/home', 'DashboardController@index')->name('home');
    Route::get('/sm/members', [MembershipController::class, 'smMembers'])->name("small.grp.members.index");
    Route::get('/export', 'DashboardController@export')->name("export");

    Route::middleware(['can:View Visit History'])->group(function () {
        Route::get('/visit/history', 'DashboardController@homepage')->name("visit.history");
    });
    Route::middleware(['is_admin'])->group(function () {
//        Route::get('/migrate', 'DashboardController@migrate')->name("migrate.companies");
        Route::get('/admin', 'DashboardController@admin')->name("companies.index");
        Route::post('/save-company', 'DashboardController@addCompany')->name("addCompany");
    });

    Route::middleware(['can:Manage Tithe'])->group(function () {

        Route::get('tithe-offerings', [TitheOfferingBaseController::class, 'index'])->name('base-offering-tithe.index');
        Route::get('tithe-offerings/{titheOfferingBase}/show', [TitheOfferingBaseController::class, 'show'])->name('base-offering-tithe.show');
        Route::delete('tithe-offerings/{titheOfferingBase}/destroy', [TitheOfferingBaseController::class, 'destroy'])->name('base-offering-tithe.destroy');
        Route::post('tithe-offerings/{offeringBase}/approve', [TitheOfferingBaseController::class, 'approve'])->name('base-offering-tithe.approve');

        Route::post('tithe-offerings/upload-cmfs-data', [TitheOfferingBaseController::class, 'uploadCmfsData'])->name('base-offering-tithe.upload-cmfs-data');
        Route::post('tithe-offerings/manual-upload', [TitheOfferingBaseController::class, 'manualUpload'])->name('base-offering-tithe.manual-upload');

        Route::get('tithe_and_offerings', [TitheAndOfferingController::class, 'index'])->name('tithe.and.offering.create');
        Route::get('tithe_and_offerings/summary/report', [TitheAndOfferingController::class, 'summaryReport'])->name('tithe.and.offering.summary.report');
        Route::get('tithe_and_offerings/detailed/report', [TitheAndOfferingController::class, 'detailedReport'])->name('tithe.and.offering.detailed.report');
        Route::get('tithe_and_offerings/summary/report/extract', [TitheAndOfferingController::class, 'summaryReportExtract'])->name('tithe.and.offering.summary.report.extract');
        Route::get('tithe_and_offerings/detailed/report/extract', [TitheAndOfferingController::class, 'detailedReportExtract'])->name('tithe.and.offering.detailed.report.extract');
        Route::post('tithe_and_offering/store', [TitheAndOfferingController::class, 'store'])->name('tithe.and.offering.store');
        Route::post('tithe_and_offering/update', [TitheAndOfferingController::class, 'update'])->name('tithe.and.offering.update');
        Route::post('tithe_and_offering/import', [TitheAndOfferingController::class, 'import'])->name('tithe.and.offering.import');
        Route::post('tithe_and_offering/cmfs/import', [TitheAndOfferingController::class, 'uploadCmfsData'])->name('cmfs.import');
        Route::post('tithe_and_offering/momo/import-preview', [TitheAndOfferingController::class, 'uploadMomoFile'])->name('tithe-and-offering.momo');
        Route::post('tithe_and_offering/momo/import', [TitheAndOfferingController::class, 'importTithes'])->name('tithe-and-offering.tithes-import');
        Route::get('tithe_and_offering/{id}/delete', [TitheAndOfferingController::class, 'destroy'])->name('tithe.and.offering.delete');
        Route::get('/get/tithe_and_offering/template', [TitheAndOfferingController::class, 'excelTemplate'])->name('tithe.and.offering.get.template');

        Route::get('/tithe_offering-export', [TitheAndOfferingController::class, 'exportToExcel'])->name('offering-and-tithe.export-to-excel');
    });

    Route::middleware(['can:Manage Users'])->group(function () {
        Route::get('/users', 'UserController@index')->name("get.users");
        Route::post('/create/user', 'UserController@save')->name("create.user");
        Route::get('users/retrieve/permission/{user}', 'UserController@permissions')->name("users.permissions.index");
        Route::post('users/add/modify/permissions', 'UserController@addPermissions')->name("users.permissions.add.modify");
    });
//Route::post('/login', 'DashboardController@login')->name("login");

    Route::middleware(['can:Manage Offering Types'])->group(function () {
//offering type
        Route::get('offering_types', [OfferingTypeController::class, 'index'])->name('offering.type.create');
        Route::post('offering_type/store', [OfferingTypeController::class, 'store'])->name('offering.store');
        Route::post('offering_type/update', [OfferingTypeController::class, 'update'])->name('offering.update');
        Route::get('offering_type/{id}/delete', [OfferingTypeController::class, 'destroy'])->name('offering.delete');
    });

    Route::middleware(['can:Manage Payment Options'])->group(function () {
        //payment option
        Route::get('payment_options', [PaymentOptionController::class, 'index'])->name('payment.option.create');
        Route::post('payment_option/store', [PaymentOptionController::class, 'store'])->name('payment.option.store');
        Route::post('payment_option/update', [PaymentOptionController::class, 'update'])->name('payment.option.update');
        Route::get('payment_option/{id}/delete', [PaymentOptionController::class, 'destroy'])->name('payment.option.delete');
    });
    Route::middleware(['can:Manage Membership'])->group(function () {
        Route::get('/members', [MembershipController::class, 'index'])->name("Membership.index");
        Route::post('/store', [MembershipController::class, 'store'])->name("Membership.store");
        Route::get('/delete/{id}', [MembershipController::class, 'destroy'])->name("Membership.delete");
        Route::get('/edit/{id}', [MembershipController::class, 'edit'])->name("Membership.edit");
        Route::put('/update/{id}', [MembershipController::class, 'update'])->name("Membership.update");
    });


    Route::middleware(['can:Manage Report Definition'])->group(function () {
        Route::get('/report_definitions', [ReportDefinitionController::class, 'index'])->name('report.definition.create');
        Route::post('/report_definition/store', [ReportDefinitionController::class, 'store'])->name('report.definition.store');
        Route::post('/report_definition/update', [ReportDefinitionController::class, 'update'])->name('report.definition.update');
        Route::get('/report_definition/{id}/delete', [ReportDefinitionController::class, 'destroy'])->name('report.definition.delete');
    });

    Route::middleware(['can:Manage Indicator Definition'])->group(function () {
        Route::get('/indicator_definitions/{definition}', [IndicatorDefinitionController::class, 'index'])->name('indicator.definition.create');
        Route::post('/indicator_definition/store', [IndicatorDefinitionController::class, 'store'])->name('indicator.definition.store');
        Route::post('/indicator_definition/update', [IndicatorDefinitionController::class, 'update'])->name('indicator.definition.update');
        Route::get('/indicator_definition/{id}/delete', [IndicatorDefinitionController::class, 'destroy'])->name('indicator.definition.delete');
    });

    Route::middleware(['can:Manage Small Groups'])->group(function () {
        Route::get('/smallgroups', [SmallGroupController::class, 'index'])->name("smallGroup.index");
        Route::post('smallgroups/store', [SmallGroupController::class, 'store'])->name("smallGroup.store");
        Route::get('smallgroups/edit/{id}', [SmallGroupController::class, 'edit'])->name("smallGroup.edit");
        Route::put('smallgroups/update', [SmallGroupController::class, 'update'])->name("smallGroup.update");
        Route::get('smallgroups/delete/{id}', [SmallGroupController::class, 'delete'])->name("smallGroup.delete");
    });

    Route::middleware(['can:Manage Expenses'])->group(function () {
        Route::get('/expenses', [ExpenseController::class, 'index'])->name("expenses.index");
        Route::post('expenses/store', [ExpenseController::class, 'store'])->name("expenses.store");
        Route::get('expenses/edit/{id}', [ExpenseController::class, 'edit'])->name("expenses.edit");
        Route::put('expenses/update', [ExpenseController::class, 'update'])->name("expenses.update");
        Route::get('expenses/delete/{id}', [ExpenseController::class, 'delete'])->name("expenses.delete");
    });

    Route::middleware(['can:Manage Expense Category'])->group(function () {
        Route::get('/expense_category', [ExpenseCategoryController::class, 'index'])->name("expensesCategory.index");
        Route::post('expense_category/store', [ExpenseCategoryController::class, 'store'])->name("expensesCategory.store");
        Route::get('expense_category/edit/{id}', [ExpenseCategoryController::class, 'edit'])->name("expensesCategory.edit");
        Route::put('expense_category/update', [ExpenseCategoryController::class, 'update'])->name("expensesCategory.update");
        Route::get('expense_category/delete/{id}', [ExpenseCategoryController::class, 'delete'])->name("expensesCategory.delete");
    });


    Route::middleware(['can:Manage Reporting Data'])->group(function () {
        Route::get('/reporting_data', [ReportingDataController::class, 'index'])->name('reporting.data.create');
        Route::get('/reporting/edit/{submission}', [ReportingDataController::class, 'edit'])->name('reporting.data.edit');
        Route::get('/reporting/submit/rpt/{submission}', [ReportingDataController::class, 'submitReport'])->name('reporting.data.submit.final');
        Route::post('/reporting/submit/data/{submission}', [ReportingDataController::class, 'submitRecord'])->name('reporting.data.submit.row');
        Route::post('/reporting_data/store', [ReportingDataController::class, 'store'])->name('reporting.data.store');
        Route::post('/reporting_data/update', [ReportingDataController::class, 'update'])->name('reporting.data.update');
        Route::get('/reporting_data/{id}/delete', [ReportingDataController::class, 'destroy'])->name('reporting.data.delete');
    });

    Route::group(['prefix' => 'reports', 'as' => 'reports.'], function () {
        Route::get('/tithe-offering', [ReportsController::class, 'titheOfferingReport'])->name('report-offering-tithe');
        Route::get('/tithe-offering-export', [ReportsController::class, 'exportTitheOfferingReport'])->name('export-tithe-offering-report');
    });
});

Auth::routes();

